#
# Copyright (C) 2007-2011 Edgewall Software, 2013-2025 the Babel team
# All rights reserved.
#
# This software is licensed as described in the file LICENSE, which
# you should have received as part of this distribution. The terms
# are also available at https://github.com/python-babel/babel/blob/master/LICENSE.
#
# This software consists of voluntary contributions made by many
# individuals. For the exact contribution history, see the revision
# history and logs, available at https://github.com/python-babel/babel/commits/master/.

import decimal
import unittest
from datetime import date

import pytest

from babel import localedata, numbers
from babel.numbers import (
    UnknownCurrencyError,
    get_currency_precision,
    get_currency_unit_pattern,
    get_decimal_precision,
    is_currency,
    list_currencies,
    normalize_currency,
    validate_currency,
)


class FormatDecimalTestCase(unittest.TestCase):

    def test_patterns(self):
        assert numbers.format_decimal(12345, '##0', locale='en_US') == '12345'
        assert numbers.format_decimal(6.5, '0.00', locale='sv') == '6,50'
        assert numbers.format_decimal((10.0 ** 20), '#.00', locale='en_US') == '100000000000000000000.00'
        # regression test for #183, fraction digits were not correctly cut
        # if the input was a float value and the value had more than 7
        # significant digits
        assert numbers.format_decimal(12345678.051, '#,##0.00', locale='en_US') == '12,345,678.05'

    def test_subpatterns(self):
        assert numbers.format_decimal((- 12345), '#,##0.##;-#', locale='en_US') == '-12,345'
        assert numbers.format_decimal((- 12345), '#,##0.##;(#)', locale='en_US') == '(12,345)'

    def test_default_rounding(self):
        """
        Testing Round-Half-Even (Banker's rounding)

        A '5' is rounded to the closest 'even' number
        """
        assert numbers.format_decimal(5.5, '0', locale='sv') == '6'
        assert numbers.format_decimal(6.5, '0', locale='sv') == '6'
        assert numbers.format_decimal(1.2325, locale='sv') == '1,232'
        assert numbers.format_decimal(1.2335, locale='sv') == '1,234'

    def test_significant_digits(self):
        """Test significant digits patterns"""
        assert numbers.format_decimal(123004, '@@', locale='en_US') == '120000'
        assert numbers.format_decimal(1.12, '@', locale='sv') == '1'
        assert numbers.format_decimal(1.1, '@@', locale='sv') == '1,1'
        assert numbers.format_decimal(1.1, '@@@@@##', locale='sv') == '1,1000'
        assert numbers.format_decimal(0.0001, '@@@', locale='sv') == '0,000100'
        assert numbers.format_decimal(0.0001234, '@@@', locale='sv') == '0,000123'
        assert numbers.format_decimal(0.0001234, '@@@#', locale='sv') == '0,0001234'
        assert numbers.format_decimal(0.0001234, '@@@#', locale='sv') == '0,0001234'
        assert numbers.format_decimal(0.12345, '@@@', locale='sv') == '0,123'
        assert numbers.format_decimal(3.14159, '@@##', locale='sv') == '3,142'
        assert numbers.format_decimal(1.23004, '@@##', locale='sv') == '1,23'
        assert numbers.format_decimal(1230.04, '@@,@@', locale='en_US') == '12,30'
        assert numbers.format_decimal(123.41, '@@##', locale='en_US') == '123.4'
        assert numbers.format_decimal(1, '@@', locale='en_US') == '1.0'
        assert numbers.format_decimal(0, '@', locale='en_US') == '0'
        assert numbers.format_decimal(0.1, '@', locale='en_US') == '0.1'
        assert numbers.format_decimal(0.1, '@#', locale='en_US') == '0.1'
        assert numbers.format_decimal(0.1, '@@', locale='en_US') == '0.10'

    def test_decimals(self):
        """Test significant digits patterns"""
        assert numbers.format_decimal(decimal.Decimal('1.2345'), '#.00', locale='en_US') == '1.23'
        assert numbers.format_decimal(decimal.Decimal('1.2345000'), '#.00', locale='en_US') == '1.23'
        assert numbers.format_decimal(decimal.Decimal('1.2345000'), '@@', locale='en_US') == '1.2'
        assert numbers.format_decimal(decimal.Decimal('12345678901234567890.12345'), '#.00', locale='en_US') == '12345678901234567890.12'

    def test_scientific_notation(self):
        assert numbers.format_scientific(0.1, '#E0', locale='en_US') == '1E-1'
        assert numbers.format_scientific(0.01, '#E0', locale='en_US') == '1E-2'
        assert numbers.format_scientific(10, '#E0', locale='en_US') == '1E1'
        assert numbers.format_scientific(1234, '0.###E0', locale='en_US') == '1.234E3'
        assert numbers.format_scientific(1234, '0.#E0', locale='en_US') == '1.2E3'
        # Exponent grouping
        assert numbers.format_scientific(12345, '##0.####E0', locale='en_US') == '1.2345E4'
        # Minimum number of int digits
        assert numbers.format_scientific(12345, '00.###E0', locale='en_US') == '12.345E3'
        assert numbers.format_scientific(-12345.6, '00.###E0', locale='en_US') == '-12.346E3'
        assert numbers.format_scientific(-0.01234, '00.###E0', locale='en_US') == '-12.34E-3'
        # Custom pattern suffix
        assert numbers.format_scientific(123.45, '#.##E0 m/s', locale='en_US') == '1.23E2 m/s'
        # Exponent patterns
        assert numbers.format_scientific(123.45, '#.##E00 m/s', locale='en_US') == '1.23E02 m/s'
        assert numbers.format_scientific(0.012345, '#.##E00 m/s', locale='en_US') == '1.23E-02 m/s'
        assert numbers.format_scientific(decimal.Decimal('12345'), '#.##E+00 m/s', locale='en_US') == '1.23E+04 m/s'
        # 0 (see ticket #99)
        assert numbers.format_scientific(0, '#E0', locale='en_US') == '0E0'

    def test_formatting_of_very_small_decimals(self):
        # previously formatting very small decimals could lead to a type error
        # because the Decimal->string conversion was too simple (see #214)
        number = decimal.Decimal("7E-7")
        assert numbers.format_decimal(number, format="@@@", locale='en_US') == '0.000000700'

    def test_nan_and_infinity(self):
        assert numbers.format_decimal(decimal.Decimal('Infinity'), locale='en_US') == '∞'
        assert numbers.format_decimal(decimal.Decimal('-Infinity'), locale='en_US') == '-∞'
        assert numbers.format_decimal(decimal.Decimal('NaN'), locale='en_US') == 'NaN'
        assert numbers.format_compact_decimal(decimal.Decimal('Infinity'), locale='en_US', format_type="short") == '∞'
        assert numbers.format_compact_decimal(decimal.Decimal('-Infinity'), locale='en_US', format_type="short") == '-∞'
        assert numbers.format_compact_decimal(decimal.Decimal('NaN'), locale='en_US', format_type="short") == 'NaN'
        assert numbers.format_currency(decimal.Decimal('Infinity'), 'USD', locale='en_US') == '$∞'
        assert numbers.format_currency(decimal.Decimal('-Infinity'), 'USD', locale='en_US') == '-$∞'

    def test_group_separator(self):
        assert numbers.format_decimal(29567.12, locale='en_US', group_separator=False) == '29567.12'
        assert numbers.format_decimal(29567.12, locale='fr_CA', group_separator=False) == '29567,12'
        assert numbers.format_decimal(29567.12, locale='pt_BR', group_separator=False) == '29567,12'
        assert numbers.format_currency(1099.98, 'USD', locale='en_US', group_separator=False) == '$1099.98'
        assert numbers.format_currency(101299.98, 'EUR', locale='fr_CA', group_separator=False) == '101299,98\xa0€'
        assert numbers.format_currency(101299.98, 'EUR', locale='en_US', group_separator=False, format_type='name') == '101299.98 euros'
        assert numbers.format_percent(251234.1234, locale='sv_SE', group_separator=False) == '25123412\xa0%'

        assert numbers.format_decimal(29567.12, locale='en_US', group_separator=True) == '29,567.12'
        assert numbers.format_decimal(29567.12, locale='fr_CA', group_separator=True) == '29\xa0567,12'
        assert numbers.format_decimal(29567.12, locale='pt_BR', group_separator=True) == '29.567,12'
        assert numbers.format_currency(1099.98, 'USD', locale='en_US', group_separator=True) == '$1,099.98'
        assert numbers.format_currency(101299.98, 'EUR', locale='fr_CA', group_separator=True) == '101\xa0299,98\xa0€'
        assert numbers.format_currency(101299.98, 'EUR', locale='en_US', group_separator=True, format_type='name') == '101,299.98 euros'
        assert numbers.format_percent(251234.1234, locale='sv_SE', group_separator=True) == '25\xa0123\xa0412\xa0%'

    def test_compact(self):
        assert numbers.format_compact_decimal(1, locale='en_US', format_type="short") == '1'
        assert numbers.format_compact_decimal(999, locale='en_US', format_type="short") == '999'
        assert numbers.format_compact_decimal(1000, locale='en_US', format_type="short") == '1K'
        assert numbers.format_compact_decimal(9000, locale='en_US', format_type="short") == '9K'
        assert numbers.format_compact_decimal(9123, locale='en_US', format_type="short", fraction_digits=2) == '9.12K'
        assert numbers.format_compact_decimal(10000, locale='en_US', format_type="short") == '10K'
        assert numbers.format_compact_decimal(10000, locale='en_US', format_type="short", fraction_digits=2) == '10K'
        assert numbers.format_compact_decimal(1000000, locale='en_US', format_type="short") == '1M'
        assert numbers.format_compact_decimal(9000999, locale='en_US', format_type="short") == '9M'
        assert numbers.format_compact_decimal(9000900099, locale='en_US', format_type="short", fraction_digits=5) == '9.0009B'
        assert numbers.format_compact_decimal(1, locale='en_US', format_type="long") == '1'
        assert numbers.format_compact_decimal(999, locale='en_US', format_type="long") == '999'
        assert numbers.format_compact_decimal(1000, locale='en_US', format_type="long") == '1 thousand'
        assert numbers.format_compact_decimal(9000, locale='en_US', format_type="long") == '9 thousand'
        assert numbers.format_compact_decimal(9000, locale='en_US', format_type="long", fraction_digits=2) == '9 thousand'
        assert numbers.format_compact_decimal(10000, locale='en_US', format_type="long") == '10 thousand'
        assert numbers.format_compact_decimal(10000, locale='en_US', format_type="long", fraction_digits=2) == '10 thousand'
        assert numbers.format_compact_decimal(1000000, locale='en_US', format_type="long") == '1 million'
        assert numbers.format_compact_decimal(9999999, locale='en_US', format_type="long") == '10 million'
        assert numbers.format_compact_decimal(9999999999, locale='en_US', format_type="long", fraction_digits=5) == '10 billion'
        assert numbers.format_compact_decimal(1, locale='ja_JP', format_type="short") == '1'
        assert numbers.format_compact_decimal(999, locale='ja_JP', format_type="short") == '999'
        assert numbers.format_compact_decimal(1000, locale='ja_JP', format_type="short") == '1000'
        assert numbers.format_compact_decimal(9123, locale='ja_JP', format_type="short") == '9123'
        assert numbers.format_compact_decimal(10000, locale='ja_JP', format_type="short") == '1万'
        assert numbers.format_compact_decimal(1234567, locale='ja_JP', format_type="short") == '123万'
        assert numbers.format_compact_decimal(-1, locale='en_US', format_type="short") == '-1'
        assert numbers.format_compact_decimal(-1234, locale='en_US', format_type="short", fraction_digits=2) == '-1.23K'
        assert numbers.format_compact_decimal(-123456789, format_type='short', locale='en_US') == '-123M'
        assert numbers.format_compact_decimal(-123456789, format_type='long', locale='en_US') == '-123 million'
        assert numbers.format_compact_decimal(2345678, locale='mk', format_type='long') == '2 милиони'
        assert numbers.format_compact_decimal(21000000, locale='mk', format_type='long') == '21 милион'
        assert numbers.format_compact_decimal(21345, locale="gv", format_type="short") == '21K'
        assert numbers.format_compact_decimal(1000, locale='it', format_type='long') == 'mille'
        assert numbers.format_compact_decimal(1234, locale='it', format_type='long') == '1 mila'
        assert numbers.format_compact_decimal(1000, locale='fr', format_type='long') == 'mille'
        assert numbers.format_compact_decimal(1234, locale='fr', format_type='long') == '1 millier'
        assert numbers.format_compact_decimal(
            12345, format_type="short", locale='ar_EG', fraction_digits=2, numbering_system='default',
        ) == '12٫34\xa0ألف'
        assert numbers.format_compact_decimal(
            12345, format_type="short", locale='ar_EG', fraction_digits=2, numbering_system='latn',
        ) == '12.34\xa0ألف'


class NumberParsingTestCase(unittest.TestCase):

    def test_can_parse_decimals(self):
        assert decimal.Decimal('1099.98') == numbers.parse_decimal('1,099.98', locale='en_US')
        assert decimal.Decimal('1099.98') == numbers.parse_decimal('1.099,98', locale='de')
        assert decimal.Decimal('1099.98') == numbers.parse_decimal('1,099.98', locale='ar', numbering_system="default")
        assert decimal.Decimal('1099.98') == numbers.parse_decimal('1٬099٫98', locale='ar_EG', numbering_system="default")
        with pytest.raises(numbers.NumberFormatError):
            numbers.parse_decimal('2,109,998', locale='de')
        with pytest.raises(numbers.UnsupportedNumberingSystemError):
            numbers.parse_decimal('2,109,998', locale='de', numbering_system="unknown")

    def test_parse_decimal_strict_mode(self):
        # Numbers with a misplaced grouping symbol should be rejected
        with pytest.raises(numbers.NumberFormatError) as info:
            numbers.parse_decimal('11.11', locale='de', strict=True)
        assert info.value.suggestions == ['1.111', '11,11']
        # Numbers with two misplaced grouping symbols should be rejected
        with pytest.raises(numbers.NumberFormatError) as info:
            numbers.parse_decimal('80.00.00', locale='de', strict=True)
        assert info.value.suggestions == ['800.000']
        # Partially grouped numbers should be rejected
        with pytest.raises(numbers.NumberFormatError) as info:
            numbers.parse_decimal('2000,000', locale='en_US', strict=True)
        assert info.value.suggestions == ['2,000,000', '2,000']
        # Numbers with duplicate grouping symbols should be rejected
        with pytest.raises(numbers.NumberFormatError) as info:
            numbers.parse_decimal('0,,000', locale='en_US', strict=True)
        assert info.value.suggestions == ['0']
        # Return only suggestion for 0 on strict
        with pytest.raises(numbers.NumberFormatError) as info:
            numbers.parse_decimal('0.00', locale='de', strict=True)
        assert info.value.suggestions == ['0']
        # Properly formatted numbers should be accepted
        assert str(numbers.parse_decimal('1.001', locale='de', strict=True)) == '1001'
        # Trailing zeroes should be accepted
        assert str(numbers.parse_decimal('3.00', locale='en_US', strict=True)) == '3.00'
        # Numbers with a grouping symbol and no trailing zeroes should be accepted
        assert str(numbers.parse_decimal('3,400.6', locale='en_US', strict=True)) == '3400.6'
        # Numbers with a grouping symbol and trailing zeroes (not all zeroes after decimal) should be accepted
        assert str(numbers.parse_decimal('3,400.60', locale='en_US', strict=True)) == '3400.60'
        # Numbers with a grouping symbol and trailing zeroes (all zeroes after decimal) should be accepted
        assert str(numbers.parse_decimal('3,400.00', locale='en_US', strict=True)) == '3400.00'
        assert str(numbers.parse_decimal('3,400.0000', locale='en_US', strict=True)) == '3400.0000'
        # Numbers with a grouping symbol and no decimal part should be accepted
        assert str(numbers.parse_decimal('3,800', locale='en_US', strict=True)) == '3800'
        # Numbers without any grouping symbol should be accepted
        assert str(numbers.parse_decimal('2000.1', locale='en_US', strict=True)) == '2000.1'
        # Numbers without any grouping symbol and no decimal should be accepted
        assert str(numbers.parse_decimal('2580', locale='en_US', strict=True)) == '2580'
        # High precision numbers should be accepted
        assert str(numbers.parse_decimal('5,000001', locale='fr', strict=True)) == '5.000001'


def test_list_currencies():
    assert isinstance(list_currencies(), set)
    assert list_currencies().issuperset(['BAD', 'BAM', 'KRO'])

    assert isinstance(list_currencies(locale='fr'), set)
    assert list_currencies('fr').issuperset(['BAD', 'BAM', 'KRO'])

    with pytest.raises(ValueError, match="expected only letters, got 'yo!'"):
        list_currencies('yo!')

    assert list_currencies(locale='pa_Arab') == {'PKR', 'INR', 'EUR'}

    assert len(list_currencies()) == 307


def test_validate_currency():
    validate_currency('EUR')

    with pytest.raises(UnknownCurrencyError, match="Unknown currency 'FUU'."):
        validate_currency('FUU')


def test_is_currency():
    assert is_currency('EUR')
    assert not is_currency('eUr')
    assert not is_currency('FUU')
    assert not is_currency('')
    assert not is_currency(None)
    assert not is_currency('   EUR    ')
    assert not is_currency('   ')
    assert not is_currency([])
    assert not is_currency(set())


def test_normalize_currency():
    assert normalize_currency('EUR') == 'EUR'
    assert normalize_currency('eUr') == 'EUR'
    assert normalize_currency('FUU') is None
    assert normalize_currency('') is None
    assert normalize_currency(None) is None
    assert normalize_currency('   EUR    ') is None
    assert normalize_currency('   ') is None
    assert normalize_currency([]) is None
    assert normalize_currency(set()) is None


def test_get_currency_name():
    assert numbers.get_currency_name('USD', locale='en_US') == 'US Dollar'
    assert numbers.get_currency_name('USD', count=2, locale='en_US') == 'US dollars'


def test_get_currency_symbol():
    assert numbers.get_currency_symbol('USD', 'en_US') == '$'


def test_get_currency_precision():
    assert get_currency_precision('EUR') == 2
    assert get_currency_precision('JPY') == 0


def test_get_currency_unit_pattern():
    assert get_currency_unit_pattern('USD', locale='en_US') == '{0} {1}'
    assert get_currency_unit_pattern('USD', locale='sw') == '{1} {0}'

    # 'ro' locale various pattern according to count
    assert get_currency_unit_pattern('USD', locale='ro', count=1) == '{0} {1}'
    assert get_currency_unit_pattern('USD', locale='ro', count=2) == '{0} {1}'
    assert get_currency_unit_pattern('USD', locale='ro', count=100) == '{0} de {1}'
    assert get_currency_unit_pattern('USD', locale='ro') == '{0} de {1}'


def test_get_territory_currencies():
    assert numbers.get_territory_currencies('AT', date(1995, 1, 1)) == ['ATS']
    assert numbers.get_territory_currencies('AT', date(2011, 1, 1)) == ['EUR']

    assert numbers.get_territory_currencies('US', date(2013, 1, 1)) == ['USD']
    assert sorted(numbers.get_territory_currencies('US', date(2013, 1, 1),
                                                   non_tender=True)) == ['USD', 'USN', 'USS']

    assert numbers.get_territory_currencies('US', date(2013, 1, 1),
                                            include_details=True) == [{
                                                'currency': 'USD',
                                                'from': date(1792, 1, 1),
                                                'to': None,
                                                'tender': True,
                                            }]

    assert numbers.get_territory_currencies('LS', date(2013, 1, 1)) == ['ZAR', 'LSL']

    assert numbers.get_territory_currencies('QO', date(2013, 1, 1)) == []

    # Croatia uses Euro starting in January 2023; this is in CLDR 42.
    # See https://github.com/python-babel/babel/issues/942
    assert 'EUR' in numbers.get_territory_currencies('HR', date(2023, 1, 1))


def test_get_decimal_symbol():
    assert numbers.get_decimal_symbol('en_US') == '.'
    assert numbers.get_decimal_symbol('en_US', numbering_system="default") == '.'
    assert numbers.get_decimal_symbol('en_US', numbering_system="latn") == '.'
    assert numbers.get_decimal_symbol('sv_SE') == ','
    assert numbers.get_decimal_symbol('ar_EG') == '.'
    assert numbers.get_decimal_symbol('ar_EG', numbering_system="default") == '٫'
    assert numbers.get_decimal_symbol('ar_EG', numbering_system="latn") == '.'
    assert numbers.get_decimal_symbol('ar_EG', numbering_system="arab") == '٫'


def test_get_plus_sign_symbol():
    assert numbers.get_plus_sign_symbol('en_US') == '+'
    assert numbers.get_plus_sign_symbol('en_US', numbering_system="default") == '+'
    assert numbers.get_plus_sign_symbol('en_US', numbering_system="latn") == '+'
    assert numbers.get_plus_sign_symbol('ar_EG') == '\u200e+'
    assert numbers.get_plus_sign_symbol('ar_EG', numbering_system="default") == '\u061c+'
    assert numbers.get_plus_sign_symbol('ar_EG', numbering_system="arab") == '\u061c+'
    assert numbers.get_plus_sign_symbol('ar_EG', numbering_system="latn") == '\u200e+'


def test_get_minus_sign_symbol():
    assert numbers.get_minus_sign_symbol('en_US') == '-'
    assert numbers.get_minus_sign_symbol('en_US', numbering_system="default") == '-'
    assert numbers.get_minus_sign_symbol('en_US', numbering_system="latn") == '-'
    assert numbers.get_minus_sign_symbol('nl_NL') == '-'
    assert numbers.get_minus_sign_symbol('ar_EG') == '\u200e-'
    assert numbers.get_minus_sign_symbol('ar_EG', numbering_system="default") == '\u061c-'
    assert numbers.get_minus_sign_symbol('ar_EG', numbering_system="arab") == '\u061c-'
    assert numbers.get_minus_sign_symbol('ar_EG', numbering_system="latn") == '\u200e-'


def test_get_exponential_symbol():
    assert numbers.get_exponential_symbol('en_US') == 'E'
    assert numbers.get_exponential_symbol('en_US', numbering_system="latn") == 'E'
    assert numbers.get_exponential_symbol('en_US', numbering_system="default") == 'E'
    assert numbers.get_exponential_symbol('ja_JP') == 'E'
    assert numbers.get_exponential_symbol('ar_EG') == 'E'
    assert numbers.get_exponential_symbol('ar_EG', numbering_system="default") == 'أس'
    assert numbers.get_exponential_symbol('ar_EG', numbering_system="arab") == 'أس'
    assert numbers.get_exponential_symbol('ar_EG', numbering_system="latn") == 'E'


def test_get_group_symbol():
    assert numbers.get_group_symbol('en_US') == ','
    assert numbers.get_group_symbol('en_US', numbering_system="latn") == ','
    assert numbers.get_group_symbol('en_US', numbering_system="default") == ','
    assert numbers.get_group_symbol('ar_EG') == ','
    assert numbers.get_group_symbol('ar_EG', numbering_system="default") == '٬'
    assert numbers.get_group_symbol('ar_EG', numbering_system="arab") == '٬'
    assert numbers.get_group_symbol('ar_EG', numbering_system="latn") == ','


def test_get_infinity_symbol():
    assert numbers.get_infinity_symbol('en_US') == '∞'
    assert numbers.get_infinity_symbol('ar_EG', numbering_system="latn") == '∞'
    assert numbers.get_infinity_symbol('ar_EG', numbering_system="default") == '∞'
    assert numbers.get_infinity_symbol('ar_EG', numbering_system="arab") == '∞'


def test_decimal_precision():
    assert get_decimal_precision(decimal.Decimal('0.110')) == 2
    assert get_decimal_precision(decimal.Decimal('1.0')) == 0
    assert get_decimal_precision(decimal.Decimal('10000')) == 0


def test_format_decimal():
    assert numbers.format_decimal(1099, locale='en_US') == '1,099'
    assert numbers.format_decimal(1099, locale='de_DE') == '1.099'
    assert numbers.format_decimal(1.2345, locale='en_US') == '1.234'
    assert numbers.format_decimal(1.2346, locale='en_US') == '1.235'
    assert numbers.format_decimal(-1.2346, locale='en_US') == '-1.235'
    assert numbers.format_decimal(1.2345, locale='sv_SE') == '1,234'
    assert numbers.format_decimal(1.2345, locale='de') == '1,234'
    assert numbers.format_decimal(12345.5, locale='en_US') == '12,345.5'
    assert numbers.format_decimal(0001.2345000, locale='en_US') == '1.234'
    assert numbers.format_decimal(-0001.2346000, locale='en_US') == '-1.235'
    assert numbers.format_decimal(0000000.5, locale='en_US') == '0.5'
    assert numbers.format_decimal(000, locale='en_US') == '0'

    assert numbers.format_decimal(12345.5, locale='ar_EG') == '12,345.5'
    assert numbers.format_decimal(12345.5, locale='ar_EG', numbering_system="default") == '12٬345٫5'
    assert numbers.format_decimal(12345.5, locale='ar_EG', numbering_system="arab") == '12٬345٫5'

    with pytest.raises(numbers.UnsupportedNumberingSystemError):
        numbers.format_decimal(12345.5, locale='en_US', numbering_system="unknown")


@pytest.mark.parametrize('input_value, expected_value', [
    ('10000', '10,000'),
    ('1', '1'),
    ('1.0', '1'),
    ('1.1', '1.1'),
    ('1.11', '1.11'),
    ('1.110', '1.11'),
    ('1.001', '1.001'),
    ('1.00100', '1.001'),
    ('01.00100', '1.001'),
    ('101.00100', '101.001'),
    ('00000', '0'),
    ('0', '0'),
    ('0.0', '0'),
    ('0.1', '0.1'),
    ('0.11', '0.11'),
    ('0.110', '0.11'),
    ('0.001', '0.001'),
    ('0.00100', '0.001'),
    ('00.00100', '0.001'),
    ('000.00100', '0.001'),
])
def test_format_decimal_precision(input_value, expected_value):
    # Test precision conservation.
    assert numbers.format_decimal(
        decimal.Decimal(input_value), locale='en_US', decimal_quantization=False) == expected_value


def test_format_decimal_quantization():
    # Test all locales.
    for locale_code in localedata.locale_identifiers():
        assert numbers.format_decimal(
            '0.9999999999', locale=locale_code, decimal_quantization=False).endswith('9999999999') is True


def test_format_currency():
    assert (numbers.format_currency(1099.98, 'USD', locale='en_US')
            == '$1,099.98')
    assert (numbers.format_currency(1099.98, 'USD', locale='en_US', numbering_system="default")
            == '$1,099.98')
    assert (numbers.format_currency(0, 'USD', locale='en_US')
            == '$0.00')
    assert (numbers.format_currency(1099.98, 'USD', locale='es_CO')
            == 'US$1.099,98')
    assert (numbers.format_currency(1099.98, 'EUR', locale='de_DE')
            == '1.099,98\xa0\u20ac')
    assert (numbers.format_currency(1099.98, 'USD', locale='ar_EG', numbering_system="default")
            == '\u200f1٬099٫98\xa0US$')
    assert (numbers.format_currency(1099.98, 'EUR', '\xa4\xa4 #,##0.00',
                                    locale='en_US')
            == 'EUR 1,099.98')
    assert (numbers.format_currency(1099.98, 'EUR', locale='nl_NL')
            != numbers.format_currency(-1099.98, 'EUR', locale='nl_NL'))
    assert (numbers.format_currency(1099.98, 'USD', format=None,
                                    locale='en_US')
            == '$1,099.98')
    assert (numbers.format_currency(1, 'USD', locale='es_AR')
            == 'US$1,00')          # one
    assert (numbers.format_currency(1000000, 'USD', locale='es_AR')
            == 'US$1.000.000,00')  # many
    assert (numbers.format_currency(0, 'USD', locale='es_AR')
            == 'US$0,00')          # other


def test_format_currency_format_type():
    assert (numbers.format_currency(1099.98, 'USD', locale='en_US',
                                    format_type="standard")
            == '$1,099.98')
    assert (numbers.format_currency(0, 'USD', locale='en_US',
                                    format_type="standard")
            == '$0.00')

    assert (numbers.format_currency(1099.98, 'USD', locale='en_US',
                                    format_type="accounting")
            == '$1,099.98')
    assert (numbers.format_currency(0, 'USD', locale='en_US',
                                    format_type="accounting")
            == '$0.00')

    with pytest.raises(numbers.UnknownCurrencyFormatError, match="'unknown' is not a known currency format type"):
        numbers.format_currency(1099.98, 'USD', locale='en_US', format_type='unknown')

    assert (numbers.format_currency(1099.98, 'JPY', locale='en_US')
            == '\xa51,100')
    assert (numbers.format_currency(1099.98, 'COP', '#,##0.00', locale='es_ES')
            == '1.099,98')
    assert (numbers.format_currency(1099.98, 'JPY', locale='en_US',
                                    currency_digits=False)
            == '\xa51,099.98')
    assert (numbers.format_currency(1099.98, 'COP', '#,##0.00', locale='es_ES',
                                    currency_digits=False)
            == '1.099,98')


def test_format_compact_currency():
    assert numbers.format_compact_currency(1, 'USD', locale='en_US', format_type="short") == '$1'
    assert numbers.format_compact_currency(999, 'USD', locale='en_US', format_type="short") == '$999'
    assert numbers.format_compact_currency(123456789, 'USD', locale='en_US', format_type="short") == '$123M'
    assert numbers.format_compact_currency(123456789, 'USD', locale='en_US', fraction_digits=2, format_type="short") == '$123.46M'
    assert numbers.format_compact_currency(123456789, 'USD', locale='en_US', fraction_digits=2, format_type="short", numbering_system="default") == '$123.46M'
    assert numbers.format_compact_currency(-123456789, 'USD', locale='en_US', fraction_digits=2, format_type="short") == '-$123.46M'
    assert numbers.format_compact_currency(1, 'JPY', locale='ja_JP', format_type="short") == '￥1'
    assert numbers.format_compact_currency(1234, 'JPY', locale='ja_JP', format_type="short") == '￥1234'
    assert numbers.format_compact_currency(123456, 'JPY', locale='ja_JP', format_type="short") == '￥12万'
    assert numbers.format_compact_currency(123456, 'JPY', locale='ja_JP', format_type="short", fraction_digits=2) == '￥12.35万'
    assert numbers.format_compact_currency(123, 'EUR', locale='yav', format_type="short") == '€\xa0123'
    assert numbers.format_compact_currency(12345, 'EUR', locale='yav', format_type="short") == '€\xa012K'
    assert numbers.format_compact_currency(123456789, 'EUR', locale='de_DE', fraction_digits=1) == '123,5\xa0Mio.\xa0€'
    assert numbers.format_compact_currency(123456789, 'USD', locale='ar_EG', fraction_digits=2, format_type="short", numbering_system="default") == '123٫46\xa0مليون\xa0US$'


def test_format_compact_currency_invalid_format_type():
    with pytest.raises(numbers.UnknownCurrencyFormatError):
        numbers.format_compact_currency(1099.98, 'USD', locale='en_US', format_type='unknown')


@pytest.mark.parametrize('input_value, expected_value', [
    ('10000', '$10,000.00'),
    ('1', '$1.00'),
    ('1.0', '$1.00'),
    ('1.1', '$1.10'),
    ('1.11', '$1.11'),
    ('1.110', '$1.11'),
    ('1.001', '$1.001'),
    ('1.00100', '$1.001'),
    ('01.00100', '$1.001'),
    ('101.00100', '$101.001'),
    ('00000', '$0.00'),
    ('0', '$0.00'),
    ('0.0', '$0.00'),
    ('0.1', '$0.10'),
    ('0.11', '$0.11'),
    ('0.110', '$0.11'),
    ('0.001', '$0.001'),
    ('0.00100', '$0.001'),
    ('00.00100', '$0.001'),
    ('000.00100', '$0.001'),
])
def test_format_currency_precision(input_value, expected_value):
    # Test precision conservation.
    assert numbers.format_currency(
        decimal.Decimal(input_value),
        currency='USD',
        locale='en_US',
        decimal_quantization=False,
    ) == expected_value


def test_format_currency_quantization():
    # Test all locales.
    for locale_code in localedata.locale_identifiers():
        assert numbers.format_currency(
            '0.9999999999', 'USD', locale=locale_code, decimal_quantization=False).find('9999999999') > -1


def test_format_currency_long_display_name():
    assert (numbers.format_currency(1099.98, 'USD', locale='en_US', format_type='name')
            == '1,099.98 US dollars')
    assert (numbers.format_currency(1099.98, 'USD', locale='en_US', format_type='name', numbering_system="default")
            == '1,099.98 US dollars')
    assert (numbers.format_currency(1099.98, 'USD', locale='ar_EG', format_type='name', numbering_system="default")
            == '1٬099٫98 دولار أمريكي')
    assert (numbers.format_currency(1.00, 'USD', locale='en_US', format_type='name')
            == '1.00 US dollar')
    assert (numbers.format_currency(1.00, 'EUR', locale='en_US', format_type='name')
            == '1.00 euro')
    assert (numbers.format_currency(2, 'EUR', locale='en_US', format_type='name')
            == '2.00 euros')
    # This tests that '{1} {0}' unitPatterns are found:
    assert (numbers.format_currency(150, 'USD', locale='sw', format_type='name')
            == 'dola za Marekani 150.00')
    assert (numbers.format_currency(1, 'USD', locale='sw', format_type='name')
            == '1.00 dola ya Marekani')
    # This tests unicode chars:
    assert (numbers.format_currency(1099.98, 'USD', locale='es_GT', format_type='name')
            == '1,099.98 dólares estadounidenses')
    # Test for completely unknown currency, should fallback to currency code
    assert (numbers.format_currency(1099.98, 'XAB', locale='en_US', format_type='name')
            == '1,099.98 XAB')

    # Test for finding different unit patterns depending on count
    assert (numbers.format_currency(1, 'USD', locale='ro', format_type='name')
            == '1,00 dolar american')
    assert (numbers.format_currency(2, 'USD', locale='ro', format_type='name')
            == '2,00 dolari americani')
    assert (numbers.format_currency(100, 'USD', locale='ro', format_type='name')
            == '100,00 de dolari americani')


def test_format_currency_long_display_name_all():
    for locale_code in localedata.locale_identifiers():
        assert numbers.format_currency(
            1, 'USD', locale=locale_code, format_type='name').find('1') > -1
        assert numbers.format_currency(
            '1', 'USD', locale=locale_code, format_type='name').find('1') > -1


def test_format_currency_long_display_name_custom_format():
    assert (numbers.format_currency(1099.98, 'USD', locale='en_US',
                                    format_type='name', format='##0')
            == '1099.98 US dollars')
    assert (numbers.format_currency(1099.98, 'USD', locale='en_US',
                                    format_type='name', format='##0',
                                    currency_digits=False)
            == '1100 US dollars')


def test_format_percent():
    assert numbers.format_percent(0.34, locale='en_US') == '34%'
    assert numbers.format_percent(0.34, locale='en_US', numbering_system="default") == '34%'
    assert numbers.format_percent(0, locale='en_US') == '0%'
    assert numbers.format_percent(0.34, '##0%', locale='en_US') == '34%'
    assert numbers.format_percent(34, '##0', locale='en_US') == '34'
    assert numbers.format_percent(25.1234, locale='en_US') == '2,512%'
    assert (numbers.format_percent(25.1234, locale='sv_SE')
            == '2\xa0512\xa0%')
    assert (numbers.format_percent(25.1234, '#,##0\u2030', locale='en_US')
            == '25,123\u2030')
    assert numbers.format_percent(134.5, locale='ar_EG', numbering_system="default") == '13٬450%'



@pytest.mark.parametrize('input_value, expected_value', [
    ('100', '10,000%'),
    ('0.01', '1%'),
    ('0.010', '1%'),
    ('0.011', '1.1%'),
    ('0.0111', '1.11%'),
    ('0.01110', '1.11%'),
    ('0.01001', '1.001%'),
    ('0.0100100', '1.001%'),
    ('0.010100100', '1.01001%'),
    ('0.000000', '0%'),
    ('0', '0%'),
    ('0.00', '0%'),
    ('0.01', '1%'),
    ('0.011', '1.1%'),
    ('0.0110', '1.1%'),
    ('0.0001', '0.01%'),
    ('0.000100', '0.01%'),
    ('0.0000100', '0.001%'),
    ('0.00000100', '0.0001%'),
])
def test_format_percent_precision(input_value, expected_value):
    # Test precision conservation.
    assert numbers.format_percent(
        decimal.Decimal(input_value), locale='en_US', decimal_quantization=False) == expected_value


def test_format_percent_quantization():
    # Test all locales.
    for locale_code in localedata.locale_identifiers():
        assert numbers.format_percent(
            '0.9999999999', locale=locale_code, decimal_quantization=False).find('99999999') > -1


def test_format_scientific():
    assert numbers.format_scientific(10000, locale='en_US') == '1E4'
    assert numbers.format_scientific(10000, locale='en_US', numbering_system="default") == '1E4'
    assert numbers.format_scientific(4234567, '#.#E0', locale='en_US') == '4.2E6'
    assert numbers.format_scientific(4234567, '0E0000', locale='en_US') == '4.234567E0006'
    assert numbers.format_scientific(4234567, '##0E00', locale='en_US') == '4.234567E06'
    assert numbers.format_scientific(4234567, '##00E00', locale='en_US') == '42.34567E05'
    assert numbers.format_scientific(4234567, '0,000E00', locale='en_US') == '4,234.567E03'
    assert numbers.format_scientific(4234567, '##0.#####E00', locale='en_US') == '4.23457E06'
    assert numbers.format_scientific(4234567, '##0.##E00', locale='en_US') == '4.23E06'
    assert numbers.format_scientific(42, '00000.000000E0000', locale='en_US') == '42000.000000E-0003'
    assert numbers.format_scientific(0.2, locale="ar_EG", numbering_system="default") == '2أس\u061c-1'


def test_default_scientific_format():
    """ Check the scientific format method auto-correct the rendering pattern
    in case of a missing fractional part.
    """
    assert numbers.format_scientific(12345, locale='en_US') == '1.2345E4'
    assert numbers.format_scientific(12345.678, locale='en_US') == '1.2345678E4'
    assert numbers.format_scientific(12345, '#E0', locale='en_US') == '1.2345E4'
    assert numbers.format_scientific(12345.678, '#E0', locale='en_US') == '1.2345678E4'


@pytest.mark.parametrize('input_value, expected_value', [
    ('10000', '1E4'),
    ('1', '1E0'),
    ('1.0', '1E0'),
    ('1.1', '1.1E0'),
    ('1.11', '1.11E0'),
    ('1.110', '1.11E0'),
    ('1.001', '1.001E0'),
    ('1.00100', '1.001E0'),
    ('01.00100', '1.001E0'),
    ('101.00100', '1.01001E2'),
    ('00000', '0E0'),
    ('0', '0E0'),
    ('0.0', '0E0'),
    ('0.1', '1E-1'),
    ('0.11', '1.1E-1'),
    ('0.110', '1.1E-1'),
    ('0.001', '1E-3'),
    ('0.00100', '1E-3'),
    ('00.00100', '1E-3'),
    ('000.00100', '1E-3'),
])
def test_format_scientific_precision(input_value, expected_value):
    # Test precision conservation.
    assert numbers.format_scientific(
        decimal.Decimal(input_value), locale='en_US', decimal_quantization=False) == expected_value


def test_format_scientific_quantization():
    # Test all locales.
    for locale_code in localedata.locale_identifiers():
        assert numbers.format_scientific(
            '0.9999999999', locale=locale_code, decimal_quantization=False).find('999999999') > -1


def test_parse_number():
    assert numbers.parse_number('1,099', locale='en_US') == 1099
    assert numbers.parse_number('1.099', locale='de_DE') == 1099
    assert numbers.parse_number('1٬099', locale='ar_EG', numbering_system="default") == 1099

    with pytest.raises(numbers.NumberFormatError, match="'1.099,98' is not a valid number"):
        numbers.parse_number('1.099,98', locale='de')

    with pytest.raises(numbers.UnsupportedNumberingSystemError):
        numbers.parse_number('1.099,98', locale='en', numbering_system="unsupported")


@pytest.mark.parametrize('string', [
    '1 099',
    '1\xa0099',
    '1\u202f099',
])
def test_parse_number_group_separator_can_be_any_space(string):
    assert numbers.parse_number(string, locale='fr') == 1099


def test_parse_decimal():
    assert (numbers.parse_decimal('1,099.98', locale='en_US')
            == decimal.Decimal('1099.98'))
    assert numbers.parse_decimal('1.099,98', locale='de') == decimal.Decimal('1099.98')

    with pytest.raises(numbers.NumberFormatError, match="'2,109,998' is not a valid decimal number"):
        numbers.parse_decimal('2,109,998', locale='de')


@pytest.mark.parametrize('string', [
    '1 099,98',
    '1\xa0099,98',
    '1\u202f099,98',
])
def test_parse_decimal_group_separator_can_be_any_space(string):
    assert decimal.Decimal('1099.98') == numbers.parse_decimal(string, locale='fr')


def test_parse_grouping():
    assert numbers.parse_grouping('##') == (1000, 1000)
    assert numbers.parse_grouping('#,###') == (3, 3)
    assert numbers.parse_grouping('#,####,###') == (3, 4)


def test_parse_pattern():

    # Original pattern is preserved
    np = numbers.parse_pattern('¤#,##0.00')
    assert np.pattern == '¤#,##0.00'

    np = numbers.parse_pattern('¤#,##0.00;(¤#,##0.00)')
    assert np.pattern == '¤#,##0.00;(¤#,##0.00)'

    # Given a NumberPattern object, we don't return a new instance.
    # However, we don't cache NumberPattern objects, so calling
    # parse_pattern with the same format string will create new
    # instances
    np1 = numbers.parse_pattern('¤ #,##0.00')
    np2 = numbers.parse_pattern('¤ #,##0.00')
    assert np1 is not np2
    assert np1 is numbers.parse_pattern(np1)


def test_parse_pattern_negative():

    # No negative format specified
    np = numbers.parse_pattern('¤#,##0.00')
    assert np.prefix == ('¤', '-¤')
    assert np.suffix == ('', '')

    # Negative format is specified
    np = numbers.parse_pattern('¤#,##0.00;(¤#,##0.00)')
    assert np.prefix == ('¤', '(¤')
    assert np.suffix == ('', ')')

    # Negative sign is a suffix
    np = numbers.parse_pattern('¤ #,##0.00;¤ #,##0.00-')
    assert np.prefix == ('¤ ', '¤ ')
    assert np.suffix == ('', '-')


def test_numberpattern_repr():
    """repr() outputs the pattern string"""

    # This implementation looks a bit funny, but that's cause strings are
    # repr'd differently in Python 2 vs 3 and this test runs under both.
    format = '¤#,##0.00;(¤#,##0.00)'
    np = numbers.parse_pattern(format)
    assert repr(format) in repr(np)


def test_parse_static_pattern():
    assert numbers.parse_pattern('Kun')  # in the So locale in CLDR 30
    # TODO: static patterns might not be correctly `apply()`ed at present


def test_parse_decimal_nbsp_heuristics():
    # Re https://github.com/python-babel/babel/issues/637 –
    #    for locales (of which there are many) that use U+00A0 as the group
    #    separator in numbers, it's reasonable to assume that input strings
    #    with plain spaces actually should have U+00A0s instead.
    #    This heuristic is only applied when strict=False.
    n = decimal.Decimal("12345.123")
    assert numbers.parse_decimal("12 345.123", locale="fi") == n
    assert numbers.parse_decimal(numbers.format_decimal(n, locale="fi"), locale="fi") == n


def test_very_small_decimal_no_quantization():
    assert numbers.format_decimal(decimal.Decimal('1E-7'), locale='en', decimal_quantization=False) == '0.0000001'


def test_single_quotes_in_pattern():
    assert numbers.format_decimal(123, "'@0.#'00'@01'", locale='en') == '@0.#120@01'

    assert numbers.format_decimal(123, "'$'''0", locale='en') == "$'123"

    assert numbers.format_decimal(12, "'#'0 o''clock", locale='en') == "#12 o'clock"


def test_format_currency_with_none_locale_with_default(monkeypatch):
    """Test that the default locale is used when locale is None."""
    monkeypatch.setattr(numbers, "LC_MONETARY", "fi_FI")
    monkeypatch.setattr(numbers, "LC_NUMERIC", None)
    assert numbers.format_currency(0, "USD", locale=None) == "0,00\xa0$"


def test_format_currency_with_none_locale(monkeypatch):
    """Test that the API raises the "Empty locale identifier" error when locale is None, and the default is too."""
    monkeypatch.setattr(numbers, "LC_MONETARY", None)  # Pretend we couldn't find any locale when importing the module
    with pytest.raises(TypeError, match="Empty"):
        numbers.format_currency(0, "USD", locale=None)


def test_format_decimal_with_none_locale_with_default(monkeypatch):
    """Test that the default locale is used when locale is None."""
    monkeypatch.setattr(numbers, "LC_NUMERIC", "fi_FI")
    monkeypatch.setattr(numbers, "LC_MONETARY", None)
    assert numbers.format_decimal("1.23", locale=None) == "1,23"


def test_format_decimal_with_none_locale(monkeypatch):
    """Test that the API raises the "Empty locale identifier" error when locale is None, and the default is too."""
    monkeypatch.setattr(numbers, "LC_NUMERIC", None)  # Pretend we couldn't find any locale when importing the module
    with pytest.raises(TypeError, match="Empty"):
        numbers.format_decimal(0, locale=None)



#
# Copyright (C) 2007-2011 Edgewall Software, 2013-2025 the Babel team
# All rights reserved.
#
# This software is licensed as described in the file LICENSE, which
# you should have received as part of this distribution. The terms
# are also available at https://github.com/python-babel/babel/blob/master/LICENSE.
#
# This software consists of voluntary contributions made by many
# individuals. For the exact contribution history, see the revision
# history and logs, available at https://github.com/python-babel/babel/commits/master/.

import unittest
from datetime import datetime
from io import BytesIO

import pytest

from babel import __version__ as VERSION
from babel.core import Locale, UnknownLocaleError
from babel.dates import format_datetime
from babel.messages import Message, checkers
from babel.messages.catalog import TranslationError
from babel.messages.checkers import _validate_format, python_format
from babel.messages.plurals import PLURALS
from babel.messages.pofile import read_po
from babel.util import LOCALTZ


class CheckersTestCase(unittest.TestCase):
    # the last msgstr[idx] is always missing except for singular plural forms

    def test_1_num_plurals_checkers(self):
        for _locale in [p for p in PLURALS if PLURALS[p][0] == 1]:
            try:
                locale = Locale.parse(_locale)
            except UnknownLocaleError:
                # Just an alias? Not what we're testing here, let's continue
                continue
            date = format_datetime(datetime.now(LOCALTZ), 'yyyy-MM-dd HH:mmZ', tzinfo=LOCALTZ, locale=_locale)
            plural = PLURALS[_locale][0]
            po_file = (f"""\
# {locale.english_name} translations for TestProject.
# Copyright (C) 2007 FooBar, Inc.
# This file is distributed under the same license as the TestProject
# project.
# FIRST AUTHOR <EMAIL@ADDRESS>, 2007.
#
msgid ""
msgstr ""
"Project-Id-Version: TestProject 0.1\\n"
"Report-Msgid-Bugs-To: bugs.address@email.tld\\n"
"POT-Creation-Date: 2007-04-01 15:30+0200\\n"
"PO-Revision-Date: {date}\\n"
"Last-Translator: FULL NAME <EMAIL@ADDRESS>\\n"
"Language-Team: {_locale} <LL@li.org>\n"
"Plural-Forms: nplurals={plural}; plural={plural};\\n"
"MIME-Version: 1.0\\n"
"Content-Type: text/plain; charset=utf-8\\n"
"Content-Transfer-Encoding: 8bit\\n"
"Generated-By: Babel {VERSION}\\n"

#. This will be a translator comment,
#. that will include several lines
#: project/file1.py:8
msgid "bar"
msgstr ""

#: project/file2.py:9
msgid "foobar"
msgid_plural "foobars"
msgstr[0] ""

""").encode('utf-8')

            # This test will fail for revisions <= 406 because so far
            # catalog.num_plurals was neglected
            catalog = read_po(BytesIO(po_file), _locale)
            message = catalog['foobar']
            checkers.num_plurals(catalog, message)

    def test_2_num_plurals_checkers(self):
        # in this testcase we add an extra msgstr[idx], we should be
        # disregarding it
        for _locale in [p for p in PLURALS if PLURALS[p][0] == 2]:
            if _locale in ['nn', 'no']:
                _locale = 'nn_NO'
                num_plurals = PLURALS[_locale.split('_')[0]][0]
                plural_expr = PLURALS[_locale.split('_')[0]][1]
            else:
                num_plurals = PLURALS[_locale][0]
                plural_expr = PLURALS[_locale][1]
            try:
                locale = Locale(_locale)
                date = format_datetime(datetime.now(LOCALTZ),
                                       'yyyy-MM-dd HH:mmZ',
                                       tzinfo=LOCALTZ, locale=_locale)
            except UnknownLocaleError:
                # Just an alias? Not what we're testing here, let's continue
                continue
            po_file = f"""\
# {locale.english_name} translations for TestProject.
# Copyright (C) 2007 FooBar, Inc.
# This file is distributed under the same license as the TestProject
# project.
# FIRST AUTHOR <EMAIL@ADDRESS>, 2007.
#
msgid ""
msgstr ""
"Project-Id-Version: TestProject 0.1\\n"
"Report-Msgid-Bugs-To: bugs.address@email.tld\\n"
"POT-Creation-Date: 2007-04-01 15:30+0200\\n"
"PO-Revision-Date: {date}\\n"
"Last-Translator: FULL NAME <EMAIL@ADDRESS>\\n"
"Language-Team: {_locale} <LL@li.org>\\n"
"Plural-Forms: nplurals={num_plurals}; plural={plural_expr};\\n"
"MIME-Version: 1.0\\n"
"Content-Type: text/plain; charset=utf-8\\n"
"Content-Transfer-Encoding: 8bit\\n"
"Generated-By: Babel {VERSION}\\n"

#. This will be a translator comment,
#. that will include several lines
#: project/file1.py:8
msgid "bar"
msgstr ""

#: project/file2.py:9
msgid "foobar"
msgid_plural "foobars"
msgstr[0] ""
msgstr[1] ""
msgstr[2] ""

""".encode('utf-8')
            # we should be adding the missing msgstr[0]

            # This test will fail for revisions <= 406 because so far
            # catalog.num_plurals was neglected
            catalog = read_po(BytesIO(po_file), _locale)
            message = catalog['foobar']
            checkers.num_plurals(catalog, message)

    def test_3_num_plurals_checkers(self):
        for _locale in [p for p in PLURALS if PLURALS[p][0] == 3]:
            plural = format_datetime(datetime.now(LOCALTZ), 'yyyy-MM-dd HH:mmZ', tzinfo=LOCALTZ, locale=_locale)
            english_name = Locale.parse(_locale).english_name
            po_file = fr"""\
# {english_name} translations for TestProject.
# Copyright (C) 2007 FooBar, Inc.
# This file is distributed under the same license as the TestProject
# project.
# FIRST AUTHOR <EMAIL@ADDRESS>, 2007.
#
msgid ""
msgstr ""
"Project-Id-Version: TestProject 0.1\n"
"Report-Msgid-Bugs-To: bugs.address@email.tld\n"
"POT-Creation-Date: 2007-04-01 15:30+0200\n"
"PO-Revision-Date: {plural}\n"
"Last-Translator: FULL NAME <EMAIL@ADDRESS>\n"
"Language-Team: {_locale} <LL@li.org>\n"
"Plural-Forms: nplurals={PLURALS[_locale][0]}; plural={PLURALS[_locale][0]};\n"
"MIME-Version: 1.0\n"
"Content-Type: text/plain; charset=utf-8\n"
"Content-Transfer-Encoding: 8bit\n"
"Generated-By: Babel {VERSION}\n"

#. This will be a translator comment,
#. that will include several lines
#: project/file1.py:8
msgid "bar"
msgstr ""

#: project/file2.py:9
msgid "foobar"
msgid_plural "foobars"
msgstr[0] ""
msgstr[1] ""

""".encode('utf-8')

            # This test will fail for revisions <= 406 because so far
            # catalog.num_plurals was neglected
            catalog = read_po(BytesIO(po_file), _locale)
            message = catalog['foobar']
            checkers.num_plurals(catalog, message)

    def test_4_num_plurals_checkers(self):
        for _locale in [p for p in PLURALS if PLURALS[p][0] == 4]:
            date = format_datetime(datetime.now(LOCALTZ), 'yyyy-MM-dd HH:mmZ', tzinfo=LOCALTZ, locale=_locale)
            english_name = Locale.parse(_locale).english_name
            plural = PLURALS[_locale][0]
            po_file = fr"""\
# {english_name} translations for TestProject.
# Copyright (C) 2007 FooBar, Inc.
# This file is distributed under the same license as the TestProject
# project.
# FIRST AUTHOR <EMAIL@ADDRESS>, 2007.
#
msgid ""
msgstr ""
"Project-Id-Version: TestProject 0.1\n"
"Report-Msgid-Bugs-To: bugs.address@email.tld\n"
"POT-Creation-Date: 2007-04-01 15:30+0200\n"
"PO-Revision-Date: {date}\n"
"Last-Translator: FULL NAME <EMAIL@ADDRESS>\n"
"Language-Team: {_locale} <LL@li.org>\n"
"Plural-Forms: nplurals={plural}; plural={plural};\n"
"MIME-Version: 1.0\n"
"Content-Type: text/plain; charset=utf-8\n"
"Content-Transfer-Encoding: 8bit\n"
"Generated-By: Babel {VERSION}\n"

#. This will be a translator comment,
#. that will include several lines
#: project/file1.py:8
msgid "bar"
msgstr ""

#: project/file2.py:9
msgid "foobar"
msgid_plural "foobars"
msgstr[0] ""
msgstr[1] ""
msgstr[2] ""

""".encode('utf-8')

            # This test will fail for revisions <= 406 because so far
            # catalog.num_plurals was neglected
            catalog = read_po(BytesIO(po_file), _locale)
            message = catalog['foobar']
            checkers.num_plurals(catalog, message)

    def test_5_num_plurals_checkers(self):
        for _locale in [p for p in PLURALS if PLURALS[p][0] == 5]:
            date = format_datetime(datetime.now(LOCALTZ), 'yyyy-MM-dd HH:mmZ', tzinfo=LOCALTZ, locale=_locale)
            english_name = Locale.parse(_locale).english_name
            plural = PLURALS[_locale][0]
            po_file = fr"""\
# {english_name} translations for TestProject.
# Copyright (C) 2007 FooBar, Inc.
# This file is distributed under the same license as the TestProject
# project.
# FIRST AUTHOR <EMAIL@ADDRESS>, 2007.
#
msgid ""
msgstr ""
"Project-Id-Version: TestProject 0.1\n"
"Report-Msgid-Bugs-To: bugs.address@email.tld\n"
"POT-Creation-Date: 2007-04-01 15:30+0200\n"
"PO-Revision-Date: {date}\n"
"Last-Translator: FULL NAME <EMAIL@ADDRESS>\n"
"Language-Team: {_locale} <LL@li.org>\n"
"Plural-Forms: nplurals={plural}; plural={plural};\n"
"MIME-Version: 1.0\n"
"Content-Type: text/plain; charset=utf-8\n"
"Content-Transfer-Encoding: 8bit\n"
"Generated-By: Babel {VERSION}\n"

#. This will be a translator comment,
#. that will include several lines
#: project/file1.py:8
msgid "bar"
msgstr ""

#: project/file2.py:9
msgid "foobar"
msgid_plural "foobars"
msgstr[0] ""
msgstr[1] ""
msgstr[2] ""
msgstr[3] ""

""".encode('utf-8')

            # This test will fail for revisions <= 406 because so far
            # catalog.num_plurals was neglected
            catalog = read_po(BytesIO(po_file), _locale)
            message = catalog['foobar']
            checkers.num_plurals(catalog, message)

    def test_6_num_plurals_checkers(self):
        for _locale in [p for p in PLURALS if PLURALS[p][0] == 6]:
            english_name = Locale.parse(_locale).english_name
            date = format_datetime(datetime.now(LOCALTZ), 'yyyy-MM-dd HH:mmZ', tzinfo=LOCALTZ, locale=_locale)
            plural = PLURALS[_locale][0]
            po_file = fr"""\
# {english_name} translations for TestProject.
# Copyright (C) 2007 FooBar, Inc.
# This file is distributed under the same license as the TestProject
# project.
# FIRST AUTHOR <EMAIL@ADDRESS>, 2007.
#
msgid ""
msgstr ""
"Project-Id-Version: TestProject 0.1\n"
"Report-Msgid-Bugs-To: bugs.address@email.tld\n"
"POT-Creation-Date: 2007-04-01 15:30+0200\n"
"PO-Revision-Date: {date}\n"
"Last-Translator: FULL NAME <EMAIL@ADDRESS>\n"
"Language-Team: {_locale} <LL@li.org>\n"
"Plural-Forms: nplurals={plural}; plural={plural};\n"
"MIME-Version: 1.0\n"
"Content-Type: text/plain; charset=utf-8\n"
"Content-Transfer-Encoding: 8bit\n"
"Generated-By: Babel {VERSION}\n"

#. This will be a translator comment,
#. that will include several lines
#: project/file1.py:8
msgid "bar"
msgstr ""

#: project/file2.py:9
msgid "foobar"
msgid_plural "foobars"
msgstr[0] ""
msgstr[1] ""
msgstr[2] ""
msgstr[3] ""
msgstr[4] ""

""".encode('utf-8')

            # This test will fail for revisions <= 406 because so far
            # catalog.num_plurals was neglected
            catalog = read_po(BytesIO(po_file), _locale)
            message = catalog['foobar']
            checkers.num_plurals(catalog, message)


class TestPythonFormat:
    @pytest.mark.parametrize(('msgid', 'msgstr'), [
        ('foo %s', 'foo'),
        (('foo %s', 'bar'), ('foo', 'bar')),
        (('foo', 'bar %s'), ('foo', 'bar')),
        (('foo %s', 'bar'), ('foo')),
    ])
    def test_python_format_invalid(self, msgid, msgstr):
        msg = Message(msgid, msgstr)
        with pytest.raises(TranslationError):
            python_format(None, msg)

    @pytest.mark.parametrize(('msgid', 'msgstr'), [
        ('foo', 'foo'),
        ('foo', 'foo %s'),
        (('foo %s', 'bar %d'), ('foo %s', 'bar %d')),
        (('foo %s', 'bar %d'), ('foo %s', 'bar %d', 'baz')),
        (('foo', 'bar %s'), ('foo')),
    ])
    def test_python_format_valid(self, msgid, msgstr):
        msg = Message(msgid, msgstr)
        python_format(None, msg)

    @pytest.mark.parametrize(('msgid', 'msgstr', 'error'), [
        ('%s %(foo)s', '%s %(foo)s', 'format string mixes positional and named placeholders'),
        ('foo %s', 'foo', 'placeholders are incompatible'),
        ('%s', '%(foo)s', 'the format strings are of different kinds'),
        ('%s', '%s %d', 'positional format placeholders are unbalanced'),
        ('%s', '%d', "incompatible format for placeholder 1: 's' and 'd' are not compatible"),
        ('%s %s %d', '%s %s %s', "incompatible format for placeholder 3: 'd' and 's' are not compatible"),
        ('%(foo)s', '%(bar)s', "unknown named placeholder 'bar'"),
        ('%(foo)s', '%(bar)d', "unknown named placeholder 'bar'"),
        ('%(foo)s', '%(foo)d', "incompatible format for placeholder 'foo': 'd' and 's' are not compatible"),
    ])
    def test__validate_format_invalid(self, msgid, msgstr, error):
        with pytest.raises(TranslationError, match=error):
            _validate_format(msgid, msgstr)

    @pytest.mark.parametrize(('msgid', 'msgstr'), [
        ('foo', 'foo'),
        ('foo', 'foo %s'),
        ('%s foo', 'foo %s'),
        ('%i', '%d'),
        ('%d', '%u'),
        ('%x', '%X'),
        ('%f', '%F'),
        ('%F', '%g'),
        ('%g', '%G'),
        ('%(foo)s', 'foo'),
        ('%(foo)s', '%(foo)s %(foo)s'),
        ('%(bar)s foo %(n)d', '%(n)d foo %(bar)s'),
    ])
    def test__validate_format_valid(self, msgid, msgstr):
        _validate_format(msgid, msgstr)



import pathlib
from io import BytesIO

import pytest

from babel.messages import frontend

toml_test_cases_path = pathlib.Path(__file__).parent / "toml-test-cases"
assert toml_test_cases_path.is_dir(), "toml-test-cases directory not found"


def test_toml_mapping_multiple_patterns():
    """
    Test that patterns may be specified as a list in TOML,
    and are expanded to multiple entries in the method map.
    """
    method_map, options_map = frontend._parse_mapping_toml(BytesIO(b"""
[[mappings]]
method = "python"
pattern = ["xyz/**.py", "foo/**.py"]
"""))
    assert len(method_map) == 2
    assert method_map[0] == ('xyz/**.py', 'python')
    assert method_map[1] == ('foo/**.py', 'python')


@pytest.mark.parametrize("test_case", toml_test_cases_path.glob("bad.*.toml"), ids=lambda p: p.name)
def test_bad_toml_test_case(test_case: pathlib.Path):
    """
    Test that bad TOML files raise a ValueError.
    """
    with pytest.raises(frontend.ConfigurationError):
        with test_case.open("rb") as f:
            frontend._parse_mapping_toml(
                f,
                filename=test_case.name,
                style="pyproject.toml" if "pyproject" in test_case.name else "standalone",
            )



import os

TEST_PROJECT_DISTRIBUTION_DATA = {
    "name": "TestProject",
    "version": "0.1",
    "packages": ["project"],
}
this_dir = os.path.abspath(os.path.dirname(__file__))
data_dir = os.path.join(this_dir, 'data')
project_dir = os.path.join(data_dir, 'project')
i18n_dir = os.path.join(project_dir, 'i18n')
pot_file = os.path.join(i18n_dir, 'temp.pot')



import pytest

from babel import lists, units


@pytest.mark.parametrize(('list', 'locale', 'expected'), [
    ([], 'en', ''),
    (['string'], 'en', 'string'),
    (['string1', 'string2'], 'en', 'string1 and string2'),
    (['string1', 'string2', 'string3'], 'en', 'string1, string2, and string3'),
    (['string1', 'string2', 'string3'], 'zh', 'string1、string2和string3'),
    (['string1', 'string2', 'string3', 'string4'], 'ne', 'string1,string2, string3 र string4'),
])
def test_format_list(list, locale, expected):
    assert lists.format_list(list, locale=locale) == expected


def test_format_list_error():
    with pytest.raises(ValueError):
        lists.format_list(['a', 'b', 'c'], style='orange', locale='en')


def test_issue_1098():
    one_foot = units.format_unit(1, "length-foot", length="short", locale="zh_CN")
    five_inches = units.format_unit(5, "length-inch", length="short", locale="zh_CN")
    # zh-CN does not specify the "unit" style, so we fall back to "unit-short" style.
    assert (
        lists.format_list([one_foot, five_inches], style="unit", locale="zh_CN") ==
        lists.format_list([one_foot, five_inches], style="unit-short", locale="zh_CN") ==
        # Translation verified using Google Translate. It would add more spacing, but the glyphs are correct.
        "1英尺5英寸"
    )


def test_lists_default_locale_deprecation():
    with pytest.warns(DeprecationWarning):
        _ = lists.DEFAULT_LOCALE



#
# Copyright (C) 2007-2011 Edgewall Software, 2013-2025 the Babel team
# All rights reserved.
#
# This software is licensed as described in the file LICENSE, which
# you should have received as part of this distribution. The terms
# are also available at https://github.com/python-babel/babel/blob/master/LICENSE.
#
# This software consists of voluntary contributions made by many
# individuals. For the exact contribution history, see the revision
# history and logs, available at https://github.com/python-babel/babel/commits/master/.

import copy
import datetime
import unittest
from io import StringIO

from babel.dates import UTC, format_datetime
from babel.messages import catalog, pofile
from babel.util import FixedOffsetTimezone


class MessageTestCase(unittest.TestCase):

    def test_python_format(self):
        assert catalog.PYTHON_FORMAT.search('foo %d bar')
        assert catalog.PYTHON_FORMAT.search('foo %s bar')
        assert catalog.PYTHON_FORMAT.search('foo %r bar')
        assert catalog.PYTHON_FORMAT.search('foo %(name).1f')
        assert catalog.PYTHON_FORMAT.search('foo %(name)3.3f')
        assert catalog.PYTHON_FORMAT.search('foo %(name)3f')
        assert catalog.PYTHON_FORMAT.search('foo %(name)06d')
        assert catalog.PYTHON_FORMAT.search('foo %(name)Li')
        assert catalog.PYTHON_FORMAT.search('foo %(name)#d')
        assert catalog.PYTHON_FORMAT.search('foo %(name)-4.4hs')
        assert catalog.PYTHON_FORMAT.search('foo %(name)*.3f')
        assert catalog.PYTHON_FORMAT.search('foo %(name).*f')
        assert catalog.PYTHON_FORMAT.search('foo %(name)3.*f')
        assert catalog.PYTHON_FORMAT.search('foo %(name)*.*f')
        assert catalog.PYTHON_FORMAT.search('foo %()s')

    def test_python_brace_format(self):
        assert not catalog._has_python_brace_format('')
        assert not catalog._has_python_brace_format('foo')
        assert not catalog._has_python_brace_format('{')
        assert not catalog._has_python_brace_format('}')
        assert not catalog._has_python_brace_format('{} {')
        assert not catalog._has_python_brace_format('{{}}')
        assert catalog._has_python_brace_format('{}')
        assert catalog._has_python_brace_format('foo {name}')
        assert catalog._has_python_brace_format('foo {name!s}')
        assert catalog._has_python_brace_format('foo {name!r}')
        assert catalog._has_python_brace_format('foo {name!a}')
        assert catalog._has_python_brace_format('foo {name!r:10}')
        assert catalog._has_python_brace_format('foo {name!r:10.2}')
        assert catalog._has_python_brace_format('foo {name!r:10.2f}')
        assert catalog._has_python_brace_format('foo {name!r:10.2f} {name!r:10.2f}')
        assert catalog._has_python_brace_format('foo {name!r:10.2f=}')

    def test_translator_comments(self):
        mess = catalog.Message('foo', user_comments=['Comment About `foo`'])
        assert mess.user_comments == ['Comment About `foo`']
        mess = catalog.Message('foo',
                               auto_comments=['Comment 1 About `foo`',
                                              'Comment 2 About `foo`'])
        assert mess.auto_comments == ['Comment 1 About `foo`', 'Comment 2 About `foo`']

    def test_clone_message_object(self):
        msg = catalog.Message('foo', locations=[('foo.py', 42)])
        clone = msg.clone()
        clone.locations.append(('bar.py', 42))
        assert msg.locations == [('foo.py', 42)]
        msg.flags.add('fuzzy')
        assert not clone.fuzzy and msg.fuzzy


class CatalogTestCase(unittest.TestCase):

    def test_add_returns_message_instance(self):
        cat = catalog.Catalog()
        message = cat.add('foo')
        assert message.id == 'foo'

    def test_two_messages_with_same_singular(self):
        cat = catalog.Catalog()
        cat.add('foo')
        cat.add(('foo', 'foos'))
        assert len(cat) == 1

    def test_duplicate_auto_comment(self):
        cat = catalog.Catalog()
        cat.add('foo', auto_comments=['A comment'])
        cat.add('foo', auto_comments=['A comment', 'Another comment'])
        assert cat['foo'].auto_comments == ['A comment', 'Another comment']

    def test_duplicate_user_comment(self):
        cat = catalog.Catalog()
        cat.add('foo', user_comments=['A comment'])
        cat.add('foo', user_comments=['A comment', 'Another comment'])
        assert cat['foo'].user_comments == ['A comment', 'Another comment']

    def test_duplicate_location(self):
        cat = catalog.Catalog()
        cat.add('foo', locations=[('foo.py', 1)])
        cat.add('foo', locations=[('foo.py', 1)])
        assert cat['foo'].locations == [('foo.py', 1)]

    def test_update_message_changed_to_plural(self):
        cat = catalog.Catalog()
        cat.add('foo', 'Voh')
        tmpl = catalog.Catalog()
        tmpl.add(('foo', 'foos'))
        cat.update(tmpl)
        assert cat['foo'].string == ('Voh', '')
        assert cat['foo'].fuzzy

    def test_update_message_changed_to_simple(self):
        cat = catalog.Catalog()
        cat.add('foo' 'foos', ('Voh', 'Vöhs'))
        tmpl = catalog.Catalog()
        tmpl.add('foo')
        cat.update(tmpl)
        assert cat['foo'].string == 'Voh'
        assert cat['foo'].fuzzy

    def test_update_message_updates_comments(self):
        cat = catalog.Catalog()
        cat['foo'] = catalog.Message('foo', locations=[('main.py', 5)])
        assert cat['foo'].auto_comments == []
        assert cat['foo'].user_comments == []
        # Update cat[u'foo'] with a new location and a comment
        cat['foo'] = catalog.Message('foo', locations=[('main.py', 7)],
                                     user_comments=['Foo Bar comment 1'])
        assert cat['foo'].user_comments == ['Foo Bar comment 1']
        # now add yet another location with another comment
        cat['foo'] = catalog.Message('foo', locations=[('main.py', 9)],
                                     auto_comments=['Foo Bar comment 2'])
        assert cat['foo'].auto_comments == ['Foo Bar comment 2']

    def test_update_fuzzy_matching_with_case_change(self):
        cat = catalog.Catalog()
        cat.add('FOO', 'Voh')
        cat.add('bar', 'Bahr')
        tmpl = catalog.Catalog()
        tmpl.add('foo')
        cat.update(tmpl)
        assert len(cat.obsolete) == 1
        assert 'FOO' not in cat

        assert cat['foo'].string == 'Voh'
        assert cat['foo'].fuzzy is True

    def test_update_fuzzy_matching_with_char_change(self):
        cat = catalog.Catalog()
        cat.add('fo', 'Voh')
        cat.add('bar', 'Bahr')
        tmpl = catalog.Catalog()
        tmpl.add('foo')
        cat.update(tmpl)
        assert len(cat.obsolete) == 1
        assert 'fo' not in cat

        assert cat['foo'].string == 'Voh'
        assert cat['foo'].fuzzy is True

    def test_update_fuzzy_matching_no_msgstr(self):
        cat = catalog.Catalog()
        cat.add('fo', '')
        tmpl = catalog.Catalog()
        tmpl.add('fo')
        tmpl.add('foo')
        cat.update(tmpl)
        assert 'fo' in cat
        assert 'foo' in cat

        assert cat['fo'].string == ''
        assert cat['fo'].fuzzy is False
        assert cat['foo'].string is None
        assert cat['foo'].fuzzy is False

    def test_update_fuzzy_matching_with_new_context(self):
        cat = catalog.Catalog()
        cat.add('foo', 'Voh')
        cat.add('bar', 'Bahr')
        tmpl = catalog.Catalog()
        tmpl.add('Foo', context='Menu')
        cat.update(tmpl)
        assert len(cat.obsolete) == 1
        assert 'foo' not in cat

        message = cat.get('Foo', 'Menu')
        assert message.string == 'Voh'
        assert message.fuzzy is True
        assert message.context == 'Menu'

    def test_update_fuzzy_matching_with_changed_context(self):
        cat = catalog.Catalog()
        cat.add('foo', 'Voh', context='Menu|File')
        cat.add('bar', 'Bahr', context='Menu|File')
        tmpl = catalog.Catalog()
        tmpl.add('Foo', context='Menu|Edit')
        cat.update(tmpl)
        assert len(cat.obsolete) == 1
        assert cat.get('Foo', 'Menu|File') is None

        message = cat.get('Foo', 'Menu|Edit')
        assert message.string == 'Voh'
        assert message.fuzzy is True
        assert message.context == 'Menu|Edit'

    def test_update_fuzzy_matching_no_cascading(self):
        cat = catalog.Catalog()
        cat.add('fo', 'Voh')
        cat.add('foo', 'Vohe')
        tmpl = catalog.Catalog()
        tmpl.add('fo')
        tmpl.add('foo')
        tmpl.add('fooo')
        cat.update(tmpl)
        assert 'fo' in cat
        assert 'foo' in cat

        assert cat['fo'].string == 'Voh'
        assert cat['fo'].fuzzy is False
        assert cat['foo'].string == 'Vohe'
        assert cat['foo'].fuzzy is False
        assert cat['fooo'].string == 'Vohe'
        assert cat['fooo'].fuzzy is True

    def test_update_fuzzy_matching_long_string(self):
        lipsum = "\
Lorem Ipsum is simply dummy text of the printing and typesetting \
industry. Lorem Ipsum has been the industry's standard dummy text ever \
since the 1500s, when an unknown printer took a galley of type and \
scrambled it to make a type specimen book. It has survived not only \
five centuries, but also the leap into electronic typesetting, \
remaining essentially unchanged. It was popularised in the 1960s with \
the release of Letraset sheets containing Lorem Ipsum passages, and \
more recently with desktop publishing software like Aldus PageMaker \
including versions of Lorem Ipsum."
        cat = catalog.Catalog()
        cat.add("ZZZZZZ " + lipsum, "foo")
        tmpl = catalog.Catalog()
        tmpl.add(lipsum + " ZZZZZZ")
        cat.update(tmpl)
        assert cat[lipsum + " ZZZZZZ"].fuzzy is True
        assert len(cat.obsolete) == 0

    def test_update_without_fuzzy_matching(self):
        cat = catalog.Catalog()
        cat.add('fo', 'Voh')
        cat.add('bar', 'Bahr')
        tmpl = catalog.Catalog()
        tmpl.add('foo')
        cat.update(tmpl, no_fuzzy_matching=True)
        assert len(cat.obsolete) == 2

    def test_fuzzy_matching_regarding_plurals(self):
        cat = catalog.Catalog()
        cat.add(('foo', 'foh'), ('foo', 'foh'))
        ru = copy.copy(cat)
        ru.locale = 'ru_RU'
        ru.update(cat)
        assert ru['foo'].fuzzy is True
        ru = copy.copy(cat)
        ru.locale = 'ru_RU'
        ru['foo'].string = ('foh', 'fohh', 'fohhh')
        ru.update(cat)
        assert ru['foo'].fuzzy is False

    def test_update_no_template_mutation(self):
        tmpl = catalog.Catalog()
        tmpl.add('foo')
        cat1 = catalog.Catalog()
        cat1.add('foo', 'Voh')
        cat1.update(tmpl)
        cat2 = catalog.Catalog()
        cat2.update(tmpl)

        assert cat2['foo'].string is None
        assert cat2['foo'].fuzzy is False

    def test_update_po_updates_pot_creation_date(self):
        template = catalog.Catalog()
        localized_catalog = copy.deepcopy(template)
        localized_catalog.locale = 'de_DE'
        assert template.mime_headers != localized_catalog.mime_headers
        assert template.creation_date == localized_catalog.creation_date
        template.creation_date = datetime.datetime.now() - \
            datetime.timedelta(minutes=5)
        localized_catalog.update(template)
        assert template.creation_date == localized_catalog.creation_date

    def test_update_po_ignores_pot_creation_date(self):
        template = catalog.Catalog()
        localized_catalog = copy.deepcopy(template)
        localized_catalog.locale = 'de_DE'
        assert template.mime_headers != localized_catalog.mime_headers
        assert template.creation_date == localized_catalog.creation_date
        template.creation_date = datetime.datetime.now() - \
            datetime.timedelta(minutes=5)
        localized_catalog.update(template, update_creation_date=False)
        assert template.creation_date != localized_catalog.creation_date

    def test_update_po_keeps_po_revision_date(self):
        template = catalog.Catalog()
        localized_catalog = copy.deepcopy(template)
        localized_catalog.locale = 'de_DE'
        fake_rev_date = datetime.datetime.now() - datetime.timedelta(days=5)
        localized_catalog.revision_date = fake_rev_date
        assert template.mime_headers != localized_catalog.mime_headers
        assert template.creation_date == localized_catalog.creation_date
        template.creation_date = datetime.datetime.now() - \
            datetime.timedelta(minutes=5)
        localized_catalog.update(template)
        assert localized_catalog.revision_date == fake_rev_date

    def test_stores_datetime_correctly(self):
        localized = catalog.Catalog()
        localized.locale = 'de_DE'
        localized[''] = catalog.Message('',
                                        "POT-Creation-Date: 2009-03-09 15:47-0700\n" +
                                        "PO-Revision-Date: 2009-03-09 15:47-0700\n")
        for key, value in localized.mime_headers:
            if key in ('POT-Creation-Date', 'PO-Revision-Date'):
                assert value == '2009-03-09 15:47-0700'

    def test_mime_headers_contain_same_information_as_attributes(self):
        cat = catalog.Catalog()
        cat[''] = catalog.Message('',
                                  "Last-Translator: Foo Bar <foo.bar@example.com>\n" +
                                  "Language-Team: de <de@example.com>\n" +
                                  "POT-Creation-Date: 2009-03-01 11:20+0200\n" +
                                  "PO-Revision-Date: 2009-03-09 15:47-0700\n")
        assert cat.locale is None
        mime_headers = dict(cat.mime_headers)

        assert cat.last_translator == 'Foo Bar <foo.bar@example.com>'
        assert mime_headers['Last-Translator'] == 'Foo Bar <foo.bar@example.com>'

        assert cat.language_team == 'de <de@example.com>'
        assert mime_headers['Language-Team'] == 'de <de@example.com>'

        dt = datetime.datetime(2009, 3, 9, 15, 47, tzinfo=FixedOffsetTimezone(-7 * 60))
        assert cat.revision_date == dt
        formatted_dt = format_datetime(dt, 'yyyy-MM-dd HH:mmZ', locale='en')
        assert mime_headers['PO-Revision-Date'] == formatted_dt


def test_message_fuzzy():
    assert not catalog.Message('foo').fuzzy
    msg = catalog.Message('foo', 'foo', flags=['fuzzy'])
    assert msg.fuzzy
    assert msg.id == 'foo'


def test_message_pluralizable():
    assert not catalog.Message('foo').pluralizable
    assert catalog.Message(('foo', 'bar')).pluralizable


def test_message_python_format():
    assert not catalog.Message('foo').python_format
    assert not catalog.Message(('foo', 'foo')).python_format
    assert catalog.Message('foo %(name)s bar').python_format
    assert catalog.Message(('foo %(name)s', 'foo %(name)s')).python_format


def test_message_python_brace_format():
    assert not catalog.Message('foo').python_brace_format
    assert not catalog.Message(('foo', 'foo')).python_brace_format
    assert catalog.Message('foo {name} bar').python_brace_format
    assert catalog.Message(('foo {name}', 'foo {name}')).python_brace_format


def test_catalog():
    cat = catalog.Catalog(project='Foobar', version='1.0',
                          copyright_holder='Foo Company')
    assert cat.header_comment == (
        '# Translations template for Foobar.\n'
        '# Copyright (C) %(year)d Foo Company\n'
        '# This file is distributed under the same '
        'license as the Foobar project.\n'
        '# FIRST AUTHOR <EMAIL@ADDRESS>, %(year)d.\n'
        '#') % {'year': datetime.date.today().year}

    cat = catalog.Catalog(project='Foobar', version='1.0',
                          copyright_holder='Foo Company')
    cat.header_comment = (
        '# The POT for my really cool PROJECT project.\n'
        '# Copyright (C) 1990-2003 ORGANIZATION\n'
        '# This file is distributed under the same license as the PROJECT\n'
        '# project.\n'
        '#\n')
    assert cat.header_comment == (
        '# The POT for my really cool Foobar project.\n'
        '# Copyright (C) 1990-2003 Foo Company\n'
        '# This file is distributed under the same license as the Foobar\n'
        '# project.\n'
        '#\n')


def test_catalog_mime_headers():
    created = datetime.datetime(1990, 4, 1, 15, 30, tzinfo=UTC)
    cat = catalog.Catalog(project='Foobar', version='1.0',
                          creation_date=created)
    assert cat.mime_headers == [
        ('Project-Id-Version', 'Foobar 1.0'),
        ('Report-Msgid-Bugs-To', 'EMAIL@ADDRESS'),
        ('POT-Creation-Date', '1990-04-01 15:30+0000'),
        ('PO-Revision-Date', 'YEAR-MO-DA HO:MI+ZONE'),
        ('Last-Translator', 'FULL NAME <EMAIL@ADDRESS>'),
        ('Language-Team', 'LANGUAGE <LL@li.org>'),
        ('MIME-Version', '1.0'),
        ('Content-Type', 'text/plain; charset=utf-8'),
        ('Content-Transfer-Encoding', '8bit'),
        ('Generated-By', f'Babel {catalog.VERSION}\n'),
    ]


def test_catalog_mime_headers_set_locale():
    created = datetime.datetime(1990, 4, 1, 15, 30, tzinfo=UTC)
    revised = datetime.datetime(1990, 8, 3, 12, 0, tzinfo=UTC)
    cat = catalog.Catalog(locale='de_DE', project='Foobar', version='1.0',
                          creation_date=created, revision_date=revised,
                          last_translator='John Doe <jd@example.com>',
                          language_team='de_DE <de@example.com>')
    assert cat.mime_headers == [
        ('Project-Id-Version', 'Foobar 1.0'),
        ('Report-Msgid-Bugs-To', 'EMAIL@ADDRESS'),
        ('POT-Creation-Date', '1990-04-01 15:30+0000'),
        ('PO-Revision-Date', '1990-08-03 12:00+0000'),
        ('Last-Translator', 'John Doe <jd@example.com>'),
        ('Language', 'de_DE'),
        ('Language-Team', 'de_DE <de@example.com>'),
        ('Plural-Forms', 'nplurals=2; plural=(n != 1);'),
        ('MIME-Version', '1.0'),
        ('Content-Type', 'text/plain; charset=utf-8'),
        ('Content-Transfer-Encoding', '8bit'),
        ('Generated-By', f'Babel {catalog.VERSION}\n'),
    ]


def test_catalog_mime_headers_type_coercion():
    """
    Test that mime headers' keys and values are coerced to strings
    """
    cat = catalog.Catalog(locale='de_DE', project='Foobar', version='1.0')
    # This is a strange interface in that it doesn't actually overwrite all
    # of the MIME headers, but just sets the ones that are passed in (and known).
    cat.mime_headers = {b'REPORT-MSGID-BUGS-TO': 8}.items()
    assert dict(cat.mime_headers)['Report-Msgid-Bugs-To'] == '8'


def test_catalog_num_plurals():
    assert catalog.Catalog(locale='en').num_plurals == 2
    assert catalog.Catalog(locale='ga').num_plurals == 5


def test_catalog_plural_expr():
    assert catalog.Catalog(locale='en').plural_expr == '(n != 1)'
    assert (catalog.Catalog(locale='ga').plural_expr
            == '(n==1 ? 0 : n==2 ? 1 : n>=3 && n<=6 ? 2 : n>=7 && n<=10 ? 3 : 4)')


def test_catalog_plural_forms():
    assert (catalog.Catalog(locale='en').plural_forms
            == 'nplurals=2; plural=(n != 1);')
    assert (catalog.Catalog(locale='pt_BR').plural_forms
            == 'nplurals=2; plural=(n > 1);')


def test_catalog_setitem():
    cat = catalog.Catalog()
    cat['foo'] = catalog.Message('foo')
    assert cat['foo'].id == 'foo'

    cat = catalog.Catalog()
    cat['foo'] = catalog.Message('foo', locations=[('main.py', 1)])
    assert cat['foo'].locations == [('main.py', 1)]
    cat['foo'] = catalog.Message('foo', locations=[('utils.py', 5)])
    assert cat['foo'].locations == [('main.py', 1), ('utils.py', 5)]


def test_catalog_add():
    cat = catalog.Catalog()
    foo = cat.add('foo')
    assert foo.id == 'foo'
    assert cat['foo'] is foo


def test_catalog_update():
    template = catalog.Catalog(header_comment="# A Custom Header")
    template.add('green', locations=[('main.py', 99)])
    template.add('blue', locations=[('main.py', 100)])
    template.add(('salad', 'salads'), locations=[('util.py', 42)])
    cat = catalog.Catalog(locale='de_DE')
    cat.add('blue', 'blau', locations=[('main.py', 98)])
    cat.add('head', 'Kopf', locations=[('util.py', 33)])
    cat.add(('salad', 'salads'), ('Salat', 'Salate'),
            locations=[('util.py', 38)])

    cat.update(template)
    assert len(cat) == 3

    msg1 = cat['green']
    assert not msg1.string
    assert msg1.locations == [('main.py', 99)]

    msg2 = cat['blue']
    assert msg2.string == 'blau'
    assert msg2.locations == [('main.py', 100)]

    msg3 = cat['salad']
    assert msg3.string == ('Salat', 'Salate')
    assert msg3.locations == [('util.py', 42)]

    assert 'head' not in cat
    assert list(cat.obsolete.values())[0].id == 'head'

    cat.update(template, update_header_comment=True)
    assert cat.header_comment == template.header_comment  # Header comment also gets updated


def test_datetime_parsing():
    val1 = catalog._parse_datetime_header('2006-06-28 23:24+0200')
    assert val1.year == 2006
    assert val1.month == 6
    assert val1.day == 28
    assert val1.tzinfo.zone == 'Etc/GMT+120'

    val2 = catalog._parse_datetime_header('2006-06-28 23:24')
    assert val2.year == 2006
    assert val2.month == 6
    assert val2.day == 28
    assert val2.tzinfo is None


def test_update_catalog_comments():
    # Based on https://web.archive.org/web/20100710131029/http://babel.edgewall.org/attachment/ticket/163/cat-update-comments.py

    catalog = pofile.read_po(StringIO('''
    # A user comment
    #. An auto comment
    #: main.py:1
    #, fuzzy, python-format
    msgid "foo %(name)s"
    msgstr "foo %(name)s"
    '''))

    assert all(message.user_comments and message.auto_comments for message in catalog if message.id)

    # NOTE: in the POT file, there are no comments
    template = pofile.read_po(StringIO('''
    #: main.py:1
    #, fuzzy, python-format
    msgid "bar %(name)s"
    msgstr ""
    '''))

    catalog.update(template)

    # Auto comments will be obliterated here
    assert all(message.user_comments for message in catalog if message.id)



from pathlib import Path

from _pytest.doctest import DoctestModule

collect_ignore = [
    'babel/messages/setuptools_frontend.py',
    'setup.py',
    'tests/messages/data',
]
babel_path = Path(__file__).parent / 'babel'


# Via the stdlib implementation of Path.is_relative_to in Python 3.9
def _is_relative(p1: Path, p2: Path) -> bool:
    try:
        p1.relative_to(p2)
        return True
    except ValueError:
        return False


def pytest_collect_file(file_path: Path, parent):
    if _is_relative(file_path, babel_path) and file_path.suffix == '.py':
        return DoctestModule.from_parent(parent, path=file_path)



from babel.languages import get_official_languages, get_territory_language_info


def test_official_languages():
    assert get_official_languages("FI") == ("fi", "sv")
    assert get_official_languages("SE") == ("sv",)
    assert get_official_languages("CH") == ("de", "fr", "it")
    assert get_official_languages("CH", de_facto=True) == ("de", "gsw", "fr", "it")
    assert get_official_languages("CH", regional=True) == ("de", "fr", "it", "rm")


def test_get_language_info():
    assert (
        set(get_territory_language_info("HU")) ==
        {"hu", "fr", "en", "de", "ro", "hr", "sk", "sl"}
    )



import inspect
import io
import os
import shutil
import sys
import tempfile

import pytest

from babel import support
from babel.messages import Catalog
from babel.messages.mofile import write_mo

SKIP_LGETTEXT = sys.version_info >= (3, 8)

messages1 = [
    ('foo', {'string': 'Voh'}),
    ('foo', {'string': 'VohCTX', 'context': 'foo'}),
    (('foo1', 'foos1'), {'string': ('Voh1', 'Vohs1')}),
    (('foo1', 'foos1'), {'string': ('VohCTX1', 'VohsCTX1'), 'context': 'foo'}),
]

messages2 = [
    ('foo', {'string': 'VohD'}),
    ('foo', {'string': 'VohCTXD', 'context': 'foo'}),
    (('foo1', 'foos1'), {'string': ('VohD1', 'VohsD1')}),
    (('foo1', 'foos1'), {'string': ('VohCTXD1', 'VohsCTXD1'), 'context': 'foo'}),
]


@pytest.fixture(autouse=True)
def use_en_us_locale(monkeypatch):
    # Use a locale which won't fail to run the tests
    monkeypatch.setenv('LANG', 'en_US.UTF-8')


@pytest.fixture()
def translations() -> support.Translations:
    catalog1 = Catalog(locale='en_GB', domain='messages')
    catalog2 = Catalog(locale='en_GB', domain='messages1')
    for ids, kwargs in messages1:
        catalog1.add(ids, **kwargs)
    for ids, kwargs in messages2:
        catalog2.add(ids, **kwargs)
    catalog1_fp = io.BytesIO()
    catalog2_fp = io.BytesIO()
    write_mo(catalog1_fp, catalog1)
    catalog1_fp.seek(0)
    write_mo(catalog2_fp, catalog2)
    catalog2_fp.seek(0)
    translations1 = support.Translations(catalog1_fp)
    translations2 = support.Translations(catalog2_fp, domain='messages1')
    return translations1.add(translations2, merge=False)


@pytest.fixture(scope='module')
def empty_translations() -> support.Translations:
    fp = io.BytesIO()
    write_mo(fp, Catalog(locale='de'))
    fp.seek(0)
    return support.Translations(fp=fp)


@pytest.fixture(scope='module')
def null_translations() -> support.NullTranslations:
    fp = io.BytesIO()
    write_mo(fp, Catalog(locale='de'))
    fp.seek(0)
    return support.NullTranslations(fp=fp)


def assert_equal_type_too(expected, result) -> None:
    assert expected == result
    assert type(expected) is type(result), (
        f'instance types do not match: {type(expected)!r}!={type(result)!r}'
    )


def test_pgettext(translations):
    assert_equal_type_too('Voh', translations.gettext('foo'))
    assert_equal_type_too('VohCTX', translations.pgettext('foo', 'foo'))
    assert_equal_type_too('VohCTX1', translations.pgettext('foo', 'foo1'))


def test_pgettext_fallback(translations):
    fallback = translations._fallback
    translations._fallback = support.NullTranslations()
    assert translations.pgettext('foo', 'bar') == 'bar'
    translations._fallback = fallback


def test_upgettext(translations):
    assert_equal_type_too('Voh', translations.ugettext('foo'))
    assert_equal_type_too('VohCTX', translations.upgettext('foo', 'foo'))


@pytest.mark.skipif(SKIP_LGETTEXT, reason='lgettext is deprecated')
def test_lpgettext(translations):
    assert_equal_type_too(b'Voh', translations.lgettext('foo'))
    assert_equal_type_too(b'VohCTX', translations.lpgettext('foo', 'foo'))


def test_npgettext(translations):
    assert_equal_type_too('Voh1', translations.ngettext('foo1', 'foos1', 1))
    assert_equal_type_too('Vohs1', translations.ngettext('foo1', 'foos1', 2))
    assert_equal_type_too('VohCTX1', translations.npgettext('foo', 'foo1', 'foos1', 1))
    assert_equal_type_too('VohsCTX1', translations.npgettext('foo', 'foo1', 'foos1', 2))


def test_unpgettext(translations):
    assert_equal_type_too('Voh1', translations.ungettext('foo1', 'foos1', 1))
    assert_equal_type_too('Vohs1', translations.ungettext('foo1', 'foos1', 2))
    assert_equal_type_too('VohCTX1', translations.unpgettext('foo', 'foo1', 'foos1', 1))
    assert_equal_type_too('VohsCTX1', translations.unpgettext('foo', 'foo1', 'foos1', 2))


@pytest.mark.skipif(SKIP_LGETTEXT, reason='lgettext is deprecated')
def test_lnpgettext(translations):
    assert_equal_type_too(b'Voh1', translations.lngettext('foo1', 'foos1', 1))
    assert_equal_type_too(b'Vohs1', translations.lngettext('foo1', 'foos1', 2))
    assert_equal_type_too(b'VohCTX1', translations.lnpgettext('foo', 'foo1', 'foos1', 1))
    assert_equal_type_too(b'VohsCTX1', translations.lnpgettext('foo', 'foo1', 'foos1', 2))


def test_dpgettext(translations):
    assert_equal_type_too('VohD', translations.dgettext('messages1', 'foo'))
    assert_equal_type_too('VohCTXD', translations.dpgettext('messages1', 'foo', 'foo'))


def test_dupgettext(translations):
    assert_equal_type_too('VohD', translations.dugettext('messages1', 'foo'))
    assert_equal_type_too('VohCTXD', translations.dupgettext('messages1', 'foo', 'foo'))


@pytest.mark.skipif(SKIP_LGETTEXT, reason='lgettext is deprecated')
def test_ldpgettext(translations):
    assert_equal_type_too(b'VohD', translations.ldgettext('messages1', 'foo'))
    assert_equal_type_too(b'VohCTXD', translations.ldpgettext('messages1', 'foo', 'foo'))


def test_dnpgettext(translations):
    assert_equal_type_too('VohD1', translations.dngettext('messages1', 'foo1', 'foos1', 1))
    assert_equal_type_too('VohsD1', translations.dngettext('messages1', 'foo1', 'foos1', 2))
    assert_equal_type_too('VohCTXD1', translations.dnpgettext('messages1', 'foo', 'foo1', 'foos1', 1))
    assert_equal_type_too('VohsCTXD1', translations.dnpgettext('messages1', 'foo', 'foo1', 'foos1', 2))


def test_dunpgettext(translations):
    assert_equal_type_too('VohD1', translations.dungettext('messages1', 'foo1', 'foos1', 1))
    assert_equal_type_too('VohsD1', translations.dungettext('messages1', 'foo1', 'foos1', 2))
    assert_equal_type_too('VohCTXD1', translations.dunpgettext('messages1', 'foo', 'foo1', 'foos1', 1))
    assert_equal_type_too('VohsCTXD1', translations.dunpgettext('messages1', 'foo', 'foo1', 'foos1', 2))


@pytest.mark.skipif(SKIP_LGETTEXT, reason='lgettext is deprecated')
def test_ldnpgettext(translations):
    assert_equal_type_too(b'VohD1', translations.ldngettext('messages1', 'foo1', 'foos1', 1))
    assert_equal_type_too(b'VohsD1', translations.ldngettext('messages1', 'foo1', 'foos1', 2))
    assert_equal_type_too(b'VohCTXD1', translations.ldnpgettext('messages1', 'foo', 'foo1', 'foos1', 1))
    assert_equal_type_too(b'VohsCTXD1', translations.ldnpgettext('messages1', 'foo', 'foo1', 'foos1', 2))


def test_load(translations):
    tempdir = tempfile.mkdtemp()
    try:
        messages_dir = os.path.join(tempdir, 'fr', 'LC_MESSAGES')
        os.makedirs(messages_dir)
        catalog = Catalog(locale='fr', domain='messages')
        catalog.add('foo', 'bar')
        with open(os.path.join(messages_dir, 'messages.mo'), 'wb') as f:
            write_mo(f, catalog)

        translations = support.Translations.load(tempdir, locales=('fr',), domain='messages')
        assert translations.gettext('foo') == 'bar'
    finally:
        shutil.rmtree(tempdir)


def get_gettext_method_names(obj):
    names = [name for name in dir(obj) if 'gettext' in name]
    if SKIP_LGETTEXT:
        # Remove deprecated l*gettext functions
        names = [name for name in names if not name.startswith('l')]
    return names


def test_null_translations_have_same_methods(empty_translations, null_translations):
    for name in get_gettext_method_names(empty_translations):
        assert hasattr(null_translations, name), f'NullTranslations does not provide method {name!r}'


def test_null_translations_method_signature_compatibility(empty_translations, null_translations):
    for name in get_gettext_method_names(empty_translations):
        assert (
            inspect.getfullargspec(getattr(empty_translations, name)) ==
            inspect.getfullargspec(getattr(null_translations, name))
        )


def test_null_translations_same_return_values(empty_translations, null_translations):
    data = {
        'message': 'foo',
        'domain': 'domain',
        'context': 'tests',
        'singular': 'bar',
        'plural': 'baz',
        'num': 1,
        'msgid1': 'bar',
        'msgid2': 'baz',
        'n': 1,
    }
    for name in get_gettext_method_names(empty_translations):
        method = getattr(empty_translations, name)
        null_method = getattr(null_translations, name)
        signature = inspect.getfullargspec(method)
        parameter_names = [name for name in signature.args if name != 'self']
        values = [data[name] for name in parameter_names]
        assert method(*values) == null_method(*values)


def test_catalog_merge_files():
    # Refs issues #92, #162
    t1 = support.Translations()
    assert t1.files == []
    t1._catalog['foo'] = 'bar'
    fp = io.BytesIO()
    write_mo(fp, Catalog())
    fp.seek(0)
    fp.name = 'pro.mo'
    t2 = support.Translations(fp)
    assert t2.files == ['pro.mo']
    t2._catalog['bar'] = 'quux'
    t1.merge(t2)
    assert t1.files == ['pro.mo']
    assert set(t1._catalog.keys()) == {'', 'foo', 'bar'}



#
# Copyright (C) 2007-2011 Edgewall Software, 2013-2025 the Babel team
# All rights reserved.
#
# This software is licensed as described in the file LICENSE, which
# you should have received as part of this distribution. The terms
# are also available at https://github.com/python-babel/babel/blob/master/LICENSE.
#
# This software consists of voluntary contributions made by many
# individuals. For the exact contribution history, see the revision
# history and logs, available at https://github.com/python-babel/babel/commits/master/.

import __future__

import unittest
from io import BytesIO

import pytest

from babel import util
from babel.util import parse_future_flags


class _FF:
    division = __future__.division.compiler_flag
    print_function = __future__.print_function.compiler_flag
    with_statement = __future__.with_statement.compiler_flag
    unicode_literals = __future__.unicode_literals.compiler_flag


def test_distinct():
    assert list(util.distinct([1, 2, 1, 3, 4, 4])) == [1, 2, 3, 4]
    assert list(util.distinct('foobar')) == ['f', 'o', 'b', 'a', 'r']


def test_pathmatch():
    assert util.pathmatch('**.py', 'bar.py')
    assert util.pathmatch('**.py', 'foo/bar/baz.py')
    assert not util.pathmatch('**.py', 'templates/index.html')
    assert util.pathmatch('**/templates/*.html', 'templates/index.html')
    assert not util.pathmatch('**/templates/*.html', 'templates/foo/bar.html')
    assert util.pathmatch('^foo/**.py', 'foo/bar/baz/blah.py')
    assert not util.pathmatch('^foo/**.py', 'blah/foo/bar/baz.py')
    assert util.pathmatch('./foo/**.py', 'foo/bar/baz/blah.py')
    assert util.pathmatch('./blah.py', 'blah.py')
    assert not util.pathmatch('./foo/**.py', 'blah/foo/bar/baz.py')


class FixedOffsetTimezoneTestCase(unittest.TestCase):

    def test_zone_negative_offset(self):
        assert util.FixedOffsetTimezone(-60).zone == 'Etc/GMT-60'

    def test_zone_zero_offset(self):
        assert util.FixedOffsetTimezone(0).zone == 'Etc/GMT+0'

    def test_zone_positive_offset(self):
        assert util.FixedOffsetTimezone(330).zone == 'Etc/GMT+330'


def parse_encoding(s):
    return util.parse_encoding(BytesIO(s.encode('utf-8')))


def test_parse_encoding_defined():
    assert parse_encoding('# coding: utf-8') == 'utf-8'


def test_parse_encoding_undefined():
    assert parse_encoding('') is None


def test_parse_encoding_non_ascii():
    assert parse_encoding('K\xf6ln') is None


@pytest.mark.parametrize('source, result', [
    ('''
from __future__ import print_function,
    division, with_statement,
    unicode_literals
''', _FF.print_function | _FF.division | _FF.with_statement | _FF.unicode_literals),
    ('''
from __future__ import print_function, division
print('hello')
''', _FF.print_function | _FF.division),
    ('''
from __future__ import print_function, division, unknown,,,,,
print 'hello'
''', _FF.print_function | _FF.division),
    ('''
from __future__ import (
    print_function,
    division)
''', _FF.print_function | _FF.division),
    ('''
from __future__ import \\
    print_function, \\
    division
''', _FF.print_function | _FF.division),
])
def test_parse_future(source, result):
    fp = BytesIO(source.encode('latin-1'))
    flags = parse_future_flags(fp)
    assert flags == result
